import UIKit
import MonnifyiOSSDK

public class SwiftMonnifyFlutterSdkPlugin: NSObject, FlutterPlugin {
    
    let ERROR_CODE_SDK_INITIALIZATION = "INIT_ERROR"
    let ERROR_CODE_PAYMENT_INITIALIZATION = "INIT_PAYMENT_ERROR"
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "monnify_flutter_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftMonnifyFlutterSdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        switch call.method {
        case "initialize":
            self.initialize(call, result)
            
        case "initializePayment":
            self.initializePayment(call, result)
            
        default:
            result(FlutterError(code: ERROR_CODE_SDK_INITIALIZATION,
                                message: "Request not understood",
                                details: nil))
        }
    }
    
    private func initialize(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        
        if let dict = call.arguments as? [String: Any] {
            let apiKey = (dict["apiKey"] as? String) ?? ""
            let contractCode = (dict["contractCode"] as? String) ?? ""
            let applicationMode = (dict["applicationMode"] as? String) ?? "TEST"
            Monnify.shared.setApiKey(apiKey: apiKey)
            Monnify.shared.setContractCode(contractCode: contractCode)
            if applicationMode=="TEST" {
                Monnify.shared.setApplicationMode(applicationMode: ApplicationMode.test)
            } else if applicationMode=="LIVE" {
                Monnify.shared.setApplicationMode(applicationMode: ApplicationMode.live)
            }
            
            result(true)
        }
        
        result(false)
        
    }
    
    private func initializePayment(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        
        if let dict = call.arguments as? [String: Any] {
            
            if dict["amount"] == nil {
                result(FlutterError(
                    code: ERROR_CODE_PAYMENT_INITIALIZATION,
                    message: "Amount cannot be empty",
                    details: nil))
                
            }
            if dict["paymentReference"] == nil {
                result(FlutterError(
                    code: ERROR_CODE_PAYMENT_INITIALIZATION,
                    message: "paymentReference cannot be empty",
                    details: nil))
                
            }
            if dict["customerEmail"] == nil {
                result(FlutterError(
                    code: ERROR_CODE_PAYMENT_INITIALIZATION,
                    message: "customerEmail cannot be empty",
                    details: nil))
                
            }
            var incomeSplitConfig: [SubAccountDetails]
            if dict["incomeSplitConfig"] == nil {
                incomeSplitConfig = []
            } else {
                let incomeSplitConfigDictionary = dict["incomeSplitConfig"] as! [[String: Any]]
                incomeSplitConfig = incomeSplitConfigDictionary.map {
                    SubAccountDetails.init(subAccountCode: ($0["subAccountCode"] as? String) ?? "",
                                           feePercentage: $0["feePercentage"] as? Float,
                                           splitAmount: Decimal.init(($0["splitAmount"] as? Double) ?? 0),
                                           feeBearer: $0["feeBearer"] as? Bool)
                }
            }
            
            let parameter = TransactionParameters(
                amount: Decimal.init((dict["amount"] as? Double) ?? 0),
                currencyCode: (dict["currencyCode"] as? String) ?? "NGN",
                paymentReference: (dict["paymentReference"] as? String) ?? "",
                customerEmail: (dict["customerEmail"] as? String) ?? "",
                customerName: (dict["customerName"] as? String) ?? "",
                customerMobileNumber: (dict["customerMobileNumber"] as? String) ?? "",
                paymentDescription: (dict["paymentDescription"] as? String) ?? "",
                incomeSplitConfig: incomeSplitConfig,
                tokeniseCard: (dict["tokeniseCard"] as? Bool) ?? false)
            
            if let rootViewController = UIApplication.shared.delegate?.window??.rootViewController {
                
                Monnify.shared.initializePayment(
                    withTransactionParameters: parameter,
                    presentingViewController: rootViewController,
                    onTransactionSuccessful: { result2 in
                        let response: [String: Any] = [
                            "transactionReference": result2.transactionReference as Any,
                            "transactionStatus": result2.transactionStatus.rawValue,
                            "paymentReference": result2.paymentReference as Any,
                            "paymentMethod": result2.paymentMethod?.rawValue as Any,
                            "amountPaid": result2.amountPaid as Any,
                            "amountPayable": result2.amountPayable as Any,
                        ]
                        result(response)
                })
                
            }
            
        }
        
        
    }
    
    
}
